﻿using System;
using System.Collections.Generic;
using System.Text;

using CommonMP.HYSSOP.CoreImpl;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;

using CommonMP.HYMCO.Interface;
using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.CoreImpl.Data;
using CommonMP.HYMCO.CoreImpl.Tool;

namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>ダムモデルクラス</para>
	/// </summary>
	public class DamModel : McForecastModelBase
	{
		/// <summary> 演算データ（キャスト用） </summary>
		DamModelInfo mInf = null;

		/// <summary><para>method outline:</para>
		/// <para>入力側の接続情報チェック</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = ReceiveConnectionCheck(ref csErrorInf)</para>
		/// </example>
		/// <param name="csErrorInf">エラー出力</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>=true:正常、=false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>受信するデータが自モデルが期待している情報か否かをチェックする</para>
		/// </remarks>
		protected override bool ReceiveConnectionCheck(ref McStructErrorInfo csErrorInf, long lInputDataNum, McReceiveCellDataIF[] csInputCellData)
		{
			bool bRtn = true;

			if (lInputDataNum == 0)
			{
				// 入力が無い場合 エラー
				csErrorInf.AddCheckErrorData(this.GetID(), DamModelDefine.HYDROLOGY_KINEMATIC_EDGE_KIND, "No receive data.");
				bRtn = false;
			}
			else if (lInputDataNum > 0)
			{
				for (long lP = 0; lP < lInputDataNum; lP++)
				{   // 入力データ数分

					if (DamModelDefine.IN_PATTERN_TOP.Equals(csInputCellData[lP].GetReceivePatternID()) == true)
					{
						// 問題なし
					}
					else if (DamModelDefine.IN_RAINDATA_POINT.Equals(csInputCellData[lP].GetReceivePatternID()) == true)
					{
						// 問題なし
					}
					else
					{
						// 予期しない接続
						csErrorInf.AddCheckErrorData(this.GetID(), DamModelDefine.HYDROLOGY_KINEMATIC_EDGE_KIND,
							"Unexpected receive data type. (Received from " + csInputCellData[lP].GetUpperElementID().ToString() + ")");
						bRtn = false;
					}
				}
			}
			return bRtn;
		}
		/// <summary><para>method outline:</para>
		/// <para>出力側の接続情報チェック</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = SendConnectionCheck(ref csErrorInf)</para>
		/// </example>
		/// <param name="csErrorInf">エラー出力</param>
		/// <param name="lOutputDataNum">出力情報数</param>
		/// <param name="csOutputCellData">出力情報配列</param>
		/// <returns>=true:正常、=false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>送信端子に設定されている伝送データが自モデルが期待している情報か否かをチェックする</para>
		/// </remarks>
		protected override bool SendConnectionCheck(ref McStructErrorInfo csErrorInf, long lOutputDataNum, McSendCellDataIF[] csOutputCellData)
		{
			bool bRtn = true;

			if (lOutputDataNum == 0)
			{
				// 警告（エラーではない）
				csErrorInf.AddCheckWarningData(this.GetID(), DamModelDefine.HYDROLOGY_KINEMATIC_EDGE_KIND, "No send port.");
			}
			else if (lOutputDataNum > 0)
			{
				for (long lP = 0; lP < lOutputDataNum; lP++)
				{   // 入力データ数分

					if (DamModelDefine.OUT_PATTERN_BOTTOM.Equals(csOutputCellData[lP].GetSendPatternID()) == true)
					{
						// 問題なし
					}
					else if (DamModelDefine.OUT_PATTERN_COND.Equals(csOutputCellData[lP].GetSendPatternID()) == true)
					{
					}
					else
					{
						// 予期しない接続
						
						csErrorInf.AddCheckErrorData(this.GetID(), DamModelDefine.HYDROLOGY_KINEMATIC_EDGE_KIND,
							"Unexpected send data type. (Send To " + csOutputCellData[lP].GetLowerElementID().ToString() + ")");
						bRtn = false;
					}
				}
			}

			return bRtn;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデルを初期化する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>Initialize(csInitialData)</para>
		/// </example>
		/// <param name="csInitialData">初期化設定情報</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override bool Initialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Initialize", "<<Init>>");
			// 引数で与えられたデータを　キャストしています。
			McInitialInfo csInDt = csInitialData as McInitialInfo;
			HySCellData csCell = null;

			if (csInDt != null)
			{
				csInDt.GetInfo("m_wlev", ref mInf.mWlev);
				mInf.mVol = WaterLevelToVol(mInf.mWlev);
			}

			// ダムへの流入量
			mInf.mQinNext = 0.0;
			mInf.mRainNext = 0.0;

			for (long lP = 0; lP < lInputDataNum; lP++)
			{   // 入力データ数分
				if (DamModelDefine.IN_PATTERN_TOP.Equals(csInputCellData[lP].GetReceivePatternID()) == true)
				{
					// 上流端からの流入量 (m3/s)
					csCell = csInputCellData[lP].GetInterpolatedCell(0);
					mInf.mQinNext += csInputCellData[0].Data(csCell, 0);
				}
				else if (DamModelDefine.IN_RAINDATA_POINT.Equals(csInputCellData[lP].GetReceivePatternID()))
				{
					// 雨量観測値
					csCell = csInputCellData[lP].GetInterpolatedCell(0);
					mInf.mRainNext = csInputCellData[0].Data(csCell, 0);
				}
			}
			mInf.mQoutNext = mInf.mQinNext;
			mInf.mAlertFlagCurrent = false;
			mInf.mPreReleaseFlagCurrent = false;
			mInf.mRainCurrent = mInf.mRainNext;
			mInf.mQinCurrent = mInf.mQinNext;
			mInf.mQoutCurrent = mInf.mQoutNext;
			mInf.mCondNext = 1;
			
			// パラメータから配列の初期化をする
			double NeedBeforeTime = 0.0;

			if (NeedBeforeTime < mInf.mAlertDecideBeforeTime)
			{
				NeedBeforeTime = mInf.mAlertDecideBeforeTime;
			}

			if (NeedBeforeTime < mInf.mAveragingTime)
			{
				NeedBeforeTime = mInf.mAveragingTime;
			}

			if (mInf.mWlevPreRelease > 0.0)
			{
				if (NeedBeforeTime < mInf.mPreReleaseDecideBeforeTime)
				{
					NeedBeforeTime = mInf.mPreReleaseDecideBeforeTime;
				}
			}

			long numOfPast = (long)(NeedBeforeTime / m_csDltTime.TotalSec()) + 1;
			mInf.numberOfPastRainSteps = numOfPast;
			mInf.mPastRain = new double[numOfPast];
			mInf.mPastTime = new double[numOfPast];
			mInf.mPastInflow = new double[numOfPast];

			UpdatePastData();
			// そのほか初期化
			

			return true;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデルを初期化する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>OnlineInitialize(csInitialData)</para>
		/// </example>
		/// <param name="csInitialData">初期化設定情報</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>bool</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override bool OnlineInitialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			return true;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデル演算</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>long lRtn = Calculate(ref csInputCellData)</para>
		/// </example>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>=0:正常 -1:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override long Calculate(long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Calculate", "<<Start>>");
			HySCellData csCell = null;

			// ダムへの流入量
			mInf.mQinNext = 0.0;
			mInf.mRainNext = 0.0;

			for (long lP = 0; lP < lInputDataNum; lP++)
			{   // 入力データ数分
				if (DamModelDefine.IN_PATTERN_TOP.Equals(csInputCellData[lP].GetReceivePatternID()) == true)
				{
					// 上流端からの流入量 (m3/s)
					csInputCellData[lP].SetCurrentTime(m_csSimTime + m_csDltTime);
					csCell = csInputCellData[lP].GetInterpolatedCell(0);
					mInf.mQinNext += csInputCellData[0].Data(csCell, 0);
				}
				else
				{
					// 雨量観測値
					csInputCellData[lP].SetCurrentTime(m_csSimTime + m_csDltTime);
					csCell = csInputCellData[lP].GetInterpolatedCell(0);
					mInf.mRainNext = csInputCellData[0].Data(csCell, 0);
				}
			}

			// m_csInf.mQinNext が 0.0 以下のときはおそらく欠測なので m_csInf.mQinCurrent で代用する
			if (mInf.mQinNext <= 0.0) mInf.mQinNext = mInf.mQinCurrent;

			// 読み込んだデータで過去の情報を更新しておく
			UpdatePastData();

			// dt 間の流入量
			double I = (mInf.mQinCurrent + mInf.mQinNext) * (double)m_csDltTime.TotalSec() * 0.5;
			// dt 間の流出量
			double O = mInf.mQoutCurrent * (double)m_csDltTime.TotalSec();
			// 貯水量
			mInf.mVol = mInf.mVol + I - O;

			// 次時刻の放流量を計算する前に水位を計算する
			mInf.mWlev = WaterVolToLev(mInf.mVol);

			mInf.mWlevTarget = mInf.mWlevHighest;

			// 過去のデータから通常放流量を算定する
			mInf.mQregularrelease = GetPastInflow(mInf.mAveragingTime) / mInf.mAveragingTime;

			for (int i = 0; i < mInf.mLimitedNum; i++)
			{
				// 次の計算時刻が洪水期間に含まれるかどうかチェック
				if (CompareDate(mInf.mFloodStartDate[i], (m_csSimTime + m_csDltTime)) && CompareDate((m_csSimTime + m_csDltTime), mInf.mFloodEndDate[i]))
				{
					mInf.mFloodPeriodFlag = true;
					mInf.mWlevTarget = mInf.mWlevLimited[i];
					break;
				}

			}
			mInf.mVolTarget = WaterLevelToVol(mInf.mWlevTarget);
			
			double wQout = Release();

			// 算定された流量を調整する
			// ただし書き操作の場合と水位が0で流せない場合はこの限りでない．
			if (mInf.mCondNext != 14)
			{
				// 流量増加率の指定がある場合
				// 流量増加率が m_csInf.mQmaxChangeRate を越えないようにする
				if (mInf.mQmaxChangeRateNum > 0)
				{
					if (wQout > mInf.mQoutCurrent)
					{
						int irank = 0;
						double wChangeRate = 0.0;
						// 流量増加率のどのラインに属するかを判定する
						for (irank = 0; irank < mInf.mQmaxChangeRateNum; irank++)
						{
							if (wQout < mInf.mQmaxChangeFlowRate[irank]) break;
						}

						// 指定外の流量であった
						if (irank == mInf.mQmaxChangeRateNum)
						{
							// 指定を上に越えているため、流量増加率を推定する
							wChangeRate = wQout * mInf.mQmaxChangeRate[irank - 1] / mInf.mQmaxChangeFlowRate[irank - 1];
						}
						else
						{
							wChangeRate = mInf.mQmaxChangeRate[irank];
						}

						if ((wQout - mInf.mQoutCurrent) / (double)m_csDltTime.TotalSec() * 3600.0 > wChangeRate)
						{
							wQout = mInf.mQoutCurrent + wChangeRate / 3600.0 * (double)m_csDltTime.TotalSec();
						}
					}
				}

				// 洪水流量より大きな値が来た場合、洪水流量まで下げること
				if (wQout > mInf.mQflood)
				{
					wQout = mInf.mQflood;
				}

				// 最低流量より小さな値が来た場合、最低流量まで上げること
				if (wQout < mInf.mQMinimum)
				{
					wQout = mInf.mQMinimum;
				}
			}
			else
			{
			}
			// 状態量更新
			mInf.mQoutNext = wQout;
			
			mInf.mAlertFlagCurrent = mInf.mAlertFlagNext;
			mInf.mPreReleaseFlagCurrent = mInf.mPreReleaseFlagNext;
			mInf.mQinCurrent = mInf.mQinNext;
			mInf.mQoutCurrent = mInf.mQoutNext;

			return 0;
		}

		/// <summary>
		/// 将来降雨の取得を行う
		/// </summary>
		/// <param name="GetTime">どれくらいの先までの平均降雨強度を取得するか</param>
		/// <returns></returns>
		double GetFutureRain(double GetTime)
		{
			// 将来降雨の取得は行わず、現在時間の降雨強度の2倍を返す
			return mInf.mRainNext * 2;
		}

		bool DecideAlert()
		{
			// 降水データは mm/hour で与えられているとする
			double wFutureRain = GetFutureRain(m_csDltTime.TotalSec() + mInf.mAlertDecideAfterTime) / 3600.0;
			double wPastRain = GetPastRain( mInf.mAlertDecideBeforeTime) / 3600.0;

			// wRainDecideAlert が mRainDecideAlert より大きければ洪水警戒体制をとる
			if (wFutureRain + wPastRain >= mInf.mAlertDecideRain)
			{
				return true;
			}

			// 現在の流入量が洪水流量の半分より多いときは洪水警戒体制をとる
			if (mInf.mQinNext >= mInf.mQflood * 0.5)
			{
				return true;
			}

			// 前回の計算では洪水警戒体制中だった
			if (mInf.mAlertFlagCurrent == true)
			{

				// 過去 mPeriodStopAlert 間の総降水量
				double wStopRain = GetPastRain(mInf.mAlertStopBeforeTime) / 3600.0;
				if (wStopRain >= mInf.mAlertStopRain)
				{
					return true;
				}
				
				// 現在洪水期間中 現在水位が制限水位を越えている場合は洪水警戒体制を継続する
				if (mInf.mWlev >= mInf.mWlevTarget * 1.001)
				{
					return true;
				}				
			}

			// 洪水警戒態勢を解除するときにピークカット実行フラグをもとに戻しておく
			mInf.mPeakFlag = false;
			return false;
		}

		bool DecidePreRelease()
		{
			// 予備放流に入るかどうか
			if (mInf.mPreReleaseFlagCurrent == false)
			{
				// 降水データは mm/hour与えられているとする
				double wPastRain = GetPastRain(mInf.mPreReleaseDecideBeforeTime) / 3600.0;
				double wFutureRain = GetFutureRain(mInf.mPreReleaseDecideAfterTime) / 3600.0; 

				if (wPastRain >= mInf.mPreReleaseDecideBeforeRain &&
					wFutureRain >= mInf.mPreReleaseDecideAfterRain &&
					mInf.mWlev > mInf.mWlevPreRelease) return true;
			}
			// 予備放流を解除するかどうか
			else
			{
				// 水位が予備放流水位を下回っていたら解除する
				if (mInf.mWlev <= mInf.mWlevPreRelease)
				{
					return false;
				}

				// 水位は下がってないが、流入量が洪水流量を上回ったら
				// 予備放流を停止し本放流に移行する
				if (mInf.mQinNext >= mInf.mQflood)
				{
					return false;
				}
			}

			// 以上の場合以外は前時刻の状態をキープする
			return mInf.mPreReleaseFlagCurrent;
		}


		double Operation_Default() // 1段階
		{
			double wSlowFlowOut = mInf.mQregularrelease + (mInf.mVol - mInf.mVolTarget * 0.999) / mInf.mLongTargetTime; 
			return wSlowFlowOut;
		}

		double Operation_Alert() // 1段階
		{
			return Operation_Default();
		}

		double Operation_ReadyPreRelease() // 1段階
		{
			return Operation_Default();
		}

		double Operation_PreRelease() // 1段階
		{
			double targetVol = WaterLevelToVol(mInf.mWlevPreRelease);
			double wPreReleaseFlowOut = mInf.mQregularrelease + (mInf.mVol - targetVol * 0.999) / mInf.mShortTargetTime;

			if (wPreReleaseFlowOut > mInf.mQmaxprerelease) wPreReleaseFlowOut = mInf.mQmaxprerelease;
			
			return wPreReleaseFlowOut;
		}

		double Operation_PeakCut() // 1段階
		{
			double wPeakFlowOut = mInf.mQflood;
			// 一定率一定量放流を実施することになっている場合
			if (mInf.mQFlowRate > 0.0)
			{
				wPeakFlowOut = mInf.mQpeakIn * mInf.mQFlowParameter + mInf.mQFlowRate;
				if (wPeakFlowOut > mInf.mQflood) wPeakFlowOut = mInf.mQflood;
			}
			return wPeakFlowOut;
		}

		double Operation_Irregular() // 1段階
		{
			double wIrregularFlowOut = mInf.mQinNext;// mInf.mQregularrelease + (mInf.mVol - WaterLevelToVol(mInf.mWlevIrregularRelease)) / m_csDltTime.TotalSec();
			return wIrregularFlowOut;
		}

		double Operation_PeakAfter() // 1段階
		{
			if (mInf.mVol < mInf.mVolTarget)
			{
				return Operation_Default();
			}
			else
			{
				double wFastFlowOut = mInf.mQregularrelease + (mInf.mVol - mInf.mVolTarget * 0.999) / mInf.mShortTargetTime;
				return wFastFlowOut;
			}
		}

		double Release()
		{
			// 水位が最低水位を下回っている場合は放流できない
			if (mInf.mWlev <= mInf.mWlevLowest)
			{
				mInf.mCondNext = 0;
				return 0.0;
			}
			

			// 次時刻に洪水警戒体制をとるべきかどうか調べる
			mInf.mAlertFlagNext = DecideAlert();

			

			// 洪水警戒体制中
			if (mInf.mAlertFlagNext == true)
			{
				// 予備放流を実施するダムである場合のみ
				if (mInf.mWlevPreRelease > 0.0)
				{
					// 予備放流体制をとるか解除するか決定する
					mInf.mPreReleaseFlagNext = DecidePreRelease();

					// 予備放流体制に入った時刻を記録する
					if (mInf.mPreReleaseFlagCurrent == false && mInf.mPreReleaseFlagNext == true)
					{
						mInf.mPreReleaseDecidedTime = m_csSimTime + m_csDltTime;
					}

					// 予備放流準備中
					if (mInf.mPreReleaseFlagNext == true)
					{
						if (m_csSimTime.TotalSec() + m_csDltTime.TotalSec() < mInf.mPreReleaseDecidedTime.TotalSec() + mInf.mPreReleasePrepareTime)
						{
							mInf.mCondNext = 11;
							return Operation_ReadyPreRelease();
						}
						else
						{
							mInf.mCondNext = 12;
							return Operation_PreRelease();
						}
					}
				}

				// ピークカット操作をするかどうかの流量基準は洪水流量
				double wDecideFlow = mInf.mQflood;

				// 一定率一定量放流を実施することになっている場合
				// 操作の変換点はその流量からになる
				if (mInf.mQFlowRate > 0.0)
				{
					wDecideFlow = mInf.mQFlowRate;
				}

				// 流入量が基準流量を上回っている
				if (mInf.mQinNext < wDecideFlow)
				{
					if (mInf.mPeakFlag == false)
					{
						// 洪水警戒態勢操作
						mInf.mCondNext = 10;
						return Operation_Alert();
					}
					else
					{
						// 洪水調節後操作
						mInf.mCondNext = 15;
						return Operation_PeakAfter();
					}
				}
				else
				{
					if (mInf.mWlev >= mInf.mWlevIrregularRelease)
					{
						// ただし書き操作
						mInf.mCondNext = 14;
						return Operation_Irregular();
					}

					// 洪水調節を行ったためピークフラグを立てる
					mInf.mPeakFlag = true;
					// 洪水調節操作
					mInf.mCondNext = 13;
					return Operation_PeakCut();	
				}
			}
			// 通常時操作
			else
			{
				// 制限水位にあわせるように捨てる
				// 水位が制限水位より高い場合
				if (mInf.mWlev > mInf.mWlevTarget)
				{
					mInf.mCondNext = 1;
					return Operation_Default();
				}
				// 水位が制限水位より低い場合
				else
				{
					mInf.mCondNext = 2;
					return Operation_Default();
				}
			}
		}


		double WaterLevelToVol(double aL)
		{
			double tmpV = 0.0;
			for (int i = 0; i < mInf.mIndexNum; i++)
			{
				if (aL < mInf.mLevIndex[i])
				{
					if (i == 0)
					{
						tmpV = mInf.mVolIndex[0] * aL / mInf.mLevIndex[0];
					}
					else
					{
						tmpV = mInf.mVolIndex[i - 1] + (mInf.mVolIndex[i] - mInf.mVolIndex[i - 1])
							* (aL - mInf.mLevIndex[i - 1]) / (mInf.mLevIndex[i] - mInf.mLevIndex[i - 1]);
					}
					break;
				}
			}
			if (tmpV == 0.0)
			{
				// 警告を出したうえでボリュームを推定して返す
				int imax = (int)(mInf.mIndexNum - 1);
				tmpV = mInf.mVolIndex[imax] + (mInf.mVolIndex[imax] - mInf.mVolIndex[imax - 1])
						* (aL - mInf.mLevIndex[imax]) / (mInf.mLevIndex[imax] - mInf.mLevIndex[imax - 1]);
			}
			return tmpV;
		}


		double WaterVolToLev(double aV)
		{
			double tmpL = 0.0;
			for (int i = 0; i < mInf.mIndexNum; i++)
			{
				if (aV < mInf.mVolIndex[i])
				{
					if (i == 0)
					{
						tmpL = aV * mInf.mLevIndex[0] / mInf.mVolIndex[0];
					}
					else
					{
						tmpL = mInf.mLevIndex[i - 1] + (mInf.mLevIndex[i] - mInf.mLevIndex[i - 1])
							* (aV - mInf.mVolIndex[i - 1]) / (mInf.mVolIndex[i] - mInf.mVolIndex[i - 1]);
					}
					break;
				}
			}
			if (tmpL == 0.0)
			{
				// 警告を出したうえでボリュームを推定して返す
				int imax = (int)(mInf.mIndexNum - 1);
				tmpL = mInf.mLevIndex[imax] + (mInf.mLevIndex[imax] - mInf.mLevIndex[imax - 1])
						* (aV - mInf.mVolIndex[imax]) / (mInf.mVolIndex[imax] - mInf.mVolIndex[imax - 1]);
			}
			return tmpL;
		}


		// 過去の流入量(mm)を計算する関数
		double GetPastInflow(double pastTime)
		{
			double wPast = 0;
			double totalPastTime = 0;
			for (int k = 1; k < (int)mInf.numberOfPastRainSteps; k++)
			{
				if (mInf.mPastTime[k] == 0.0) break;
				if (mInf.mPastTime[k] <= m_csSimTime.TotalSec() - pastTime) break;
				wPast += mInf.mPastInflow[k] * (mInf.mPastTime[k - 1] - mInf.mPastTime[k]);
				totalPastTime += mInf.mPastTime[k - 1] - mInf.mPastTime[k];
			}
			if (totalPastTime == 0) return mInf.mQinNext * pastTime;
			return wPast * pastTime / totalPastTime; 
		}

		// 過去の積算降雨量(mm)を計算する関数
		double GetPastRain(double pastTime) // pastTime はさかのぼる期間
		{
			double wPast = 0;
			double totalPastTime = 0;
			for (int k = 1; k < (int)mInf.numberOfPastRainSteps; k++)
			{
				if (mInf.mPastTime[k] == 0.0) break;
				if (mInf.mPastTime[k] <= m_csSimTime.TotalSec() - pastTime) break;
				wPast += mInf.mPastRain[k] * (mInf.mPastTime[k - 1] - mInf.mPastTime[k]);
				totalPastTime += mInf.mPastTime[k - 1] - mInf.mPastTime[k];
			}
			if (totalPastTime == 0) return mInf.mRainNext * pastTime;
			return wPast * pastTime / totalPastTime / 3600.0;  // (mm/h) * (3600 sec) -> (mm)
		}

		// 過去の積算降雨量、流入量を更新する関数 Change 13
		void UpdatePastData()
		{
			int k;
			for (k = (int)(mInf.numberOfPastRainSteps - 1); k > 0; k--)
			{
				mInf.mPastRain[k] = mInf.mPastRain[k - 1];
				mInf.mPastTime[k] = mInf.mPastTime[k - 1];
				mInf.mPastInflow[k] = mInf.mPastInflow[k - 1];
			}
			mInf.mPastRain[0] = mInf.mRainNext;
			mInf.mPastInflow[0] = mInf.mQinNext;
			mInf.mPastTime[0] = m_csSimTime.TotalSec();
		}

		/// <summary><para>method outline:</para>
		/// <para>モデル演算結果を外部のエレメントに対して公開する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>long lRtn = DataFusion( lOutputDataNum, ref csOutputCellData)</para>
		/// </example>
		/// <param name="lOutputDataNum">出力情報数</param>
		/// <param name="csOutputCellData">出力情報配列</param>
		/// <returns>=0:正常 -1:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override long DataFusion(long lOutputDataNum, ref McSendCellDataIF[] csOutputCellData)
		{
			
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "DataFusion", "<<Start>>");

			HySCellData[] csSndCellData = null;
			// 最終セルだけの出力
			for (long lP = 0; lP < lOutputDataNum; lP++)
			{
				if (DamModelDefine.OUT_PATTERN_BOTTOM.Equals(csOutputCellData[lP].GetSendPatternID()))
				{
					csSndCellData = csOutputCellData[lP].PrepareSendCellD1();
					csSndCellData[0].m_dData[0] = mInf.mQoutCurrent;
				}
				else if (DamModelDefine.OUT_PATTERN_COND.Equals(csOutputCellData[lP].GetSendPatternID()))
				{
					csSndCellData = csOutputCellData[lP].PrepareSendCellD1();
					csSndCellData[0].m_dData[0] = mInf.mWlev;
					csSndCellData[0].m_dData[1] = mInf.mVol;
					csSndCellData[0].m_dData[2] = mInf.mCondNext;
				}
			}
			return 0;
		}

		/// <summary>
		/// 年の単位を無視して、月日の大小だけを見る
		/// 後者のほうが後の時間である場合Trueを返す
		/// 例：tTime1:8月  tTime2:9月のときTrue
		/// 同じ日付である場合はTrue
		/// </summary>
		/// <param name="tTime1">tTime1</param>
		/// <param name="tTime2">tTime2</param>
		/// <returns>
		/// </returns>
		public bool CompareDate(HySTime tTime1, HySTime tTime2)
		{
			DateTime csDT = DateTime.Parse("1/1/1970 0:0:0");
			TimeSpan csTS = TimeSpan.FromMilliseconds(tTime1.GetTime() * 1000);
			csDT = csDT.Add(csTS);
			int m1 = csDT.Month;
			int d1 = csDT.Day;

			csDT = DateTime.Parse("1/1/1970 0:0:0");
			csTS = TimeSpan.FromMilliseconds(tTime2.GetTime() * 1000);
			csDT = csDT.Add(csTS);
			int m2 = csDT.Month;
			int d2 = csDT.Day;

			if (m1 > m2) return false;
			if (m1 == m2 && d1 > d2) return false;

			return true;
		}

		// ====================
		// その他必要なメソッド
		// ====================

		/// <summary><para>method outline:</para>
		/// <para>プロパティ情報を設定する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>SetProperty(csCellMdlPropertyInfo)</para>
		/// </example>
		/// <param name="csCellMdlPropertyInfo">セル型プロパティ情報</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override bool SetProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "<<Init>>");
			// 使用しやすいようにキャストしておく
			mInf = (DamModelInfo)m_csCalInfo;

			// プロパティ設定
			McCellModelPropertyInfo csPrptyInfo = csCellMdlPropertyInfo as McCellModelPropertyInfo;

			if (csPrptyInfo != null)
			{
				// 演算ステップ時刻設定
				this.m_csDltTime = new HySTime(csPrptyInfo.GetStepTime());

				csPrptyInfo.GetInfo("m_pmtfile", ref mInf.m_pmtfile);

				int iReadTiming = 0;
				long lMonth;
				long lDay;
				string sLine = "";
				string[] sWords;
				try
				{
					// 入力ファイル
					HySFile fpParam = new HySFile(mInf.m_pmtfile);
					if(fpParam.Open(HySFile.OPEN_MODE.OPEN, HySFile.READ_WRITE_MODE.READ, HySFile.DIRECTORY_MODE.MK_DIR) < 0)
					{
						throw new Exception("No File.");

					}

					while (true)
					{
						long lRtn = fpParam.ReadText(ref sLine);
						if (lRtn == 0 || lRtn == -1)
						{
							// ファイルの終点に達した
							break;
						}

						if (sLine.Length < 1) continue;

						// コメント行である場合は飛ばす
						if (sLine[0] != '#')
						{
							sWords = sLine.Split(new Char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);

							switch (iReadTiming)
							{
								case 0: // 水位などの基礎情報
									if (sWords.Length != 7) return false;
									mInf.mWlevSurcharge = double.Parse(sWords[0]);
									mInf.mWlevIrregularRelease = double.Parse(sWords[1]);
									mInf.mWlevHighest = double.Parse(sWords[2]);
									mInf.mWlevMinimum = double.Parse(sWords[3]);
									mInf.mWlevLowest = double.Parse(sWords[4]);
									mInf.mQflood = double.Parse(sWords[5]);
									mInf.mQMinimum = double.Parse(sWords[6]);

									iReadTiming++;
									break;

								case 1: // mFloodStartDate
									if (sWords.Length != 2) return false;
									lMonth = long.Parse(sWords[0]);
									lDay = long.Parse(sWords[1]);

									mInf.mFloodStartDate = new HySTime[2];
									mInf.mFloodEndDate = new HySTime[2];
									mInf.mWlevLimited = new double[2];

									mInf.mFloodStartDate[0] = HySCalendar.CreateTime(0, lMonth, lDay, 0, 0, 0);
									iReadTiming++;
									break;
								case 2: // mFloodEndDate
									if (sWords.Length != 2) return false;
									lMonth = long.Parse(sWords[0]);
									lDay = long.Parse(sWords[1]);
									mInf.mFloodEndDate[0] = HySCalendar.CreateTime(0, lMonth, lDay, 0, 0, 0);
									iReadTiming++;
									break;
								case 3: // mWlevLimited
									if (sWords.Length != 1) return false;
									mInf.mWlevLimited[0] = double.Parse(sWords[0]);
									iReadTiming++;
									break;

								case 4: // mFloodStartDateSecond
									if (sWords.Length != 2) return false;
									lMonth = long.Parse(sWords[0]);
									lDay = long.Parse(sWords[1]);
									mInf.mFloodStartDate[1] = HySCalendar.CreateTime(0, lMonth, lDay, 0, 0, 0);
									iReadTiming++;
									break;
								case 5: // mFloodEndDateSecond
									if (sWords.Length != 2) return false;
									lMonth = long.Parse(sWords[0]);
									lDay = long.Parse(sWords[1]);
									mInf.mFloodEndDate[1] = HySCalendar.CreateTime(0, lMonth, lDay, 0, 0, 0);
									iReadTiming++;
									break;
								case 6: // mWlevLimitedSecond
									if (sWords.Length != 1) return false;
									mInf.mWlevLimited[1] = double.Parse(sWords[0]);
									iReadTiming++;

									mInf.mQmaxChangeRateNum = 0;

									break;

								case 7:// mQmaxChangeRateと予備放流パラメータ
									if (sWords.Length != 2)
									{
										if (mInf.mQmaxChangeRateNum == 0) return false;
										if (mInf.mQmaxChangeRateNum == 1 && mInf.mQmaxChangeRate[0] == 0.0) mInf.mQmaxChangeRateNum = 0;
										if (sWords.Length != 7) return false;
										mInf.mWlevPreRelease = double.Parse(sWords[0]);
										mInf.mQmaxprerelease = double.Parse(sWords[1]);
										mInf.mPreReleaseDecideBeforeTime = double.Parse(sWords[2]);
										mInf.mPreReleaseDecideBeforeRain = double.Parse(sWords[3]);
										mInf.mPreReleaseDecideAfterTime = double.Parse(sWords[4]);
										mInf.mPreReleaseDecideAfterRain = double.Parse(sWords[5]);
										mInf.mPreReleasePrepareTime = double.Parse(sWords[6]);
										iReadTiming++;
										break;
									}

									if (mInf.mQmaxChangeRateNum >= 10) return false;
									mInf.mQmaxChangeRate[mInf.mQmaxChangeRateNum] = double.Parse(sWords[0]);
									mInf.mQmaxChangeFlowRate[mInf.mQmaxChangeRateNum] = double.Parse(sWords[1]);
									mInf.mQmaxChangeRateNum++;
									break;

								case 8: // 一定量一定率放流パラメータ
									if (sWords.Length != 2) return false;
									mInf.mQFlowRate = double.Parse(sWords[0]);
									mInf.mQFlowParameter = double.Parse(sWords[1]);
									iReadTiming++;

									mInf.mIndexNum = 0;

									break;

								case 9: // 水位流量関係(水位 流量)と洪水警戒態勢パラメータ
									if (sWords.Length != 2)
									{
										if (mInf.mIndexNum <= 2) return false;
										if (sWords.Length != 6) return false;

										mInf.mAlertDecideBeforeTime = double.Parse(sWords[0]);
										mInf.mAlertDecideAfterTime = double.Parse(sWords[1]);
										mInf.mAlertDecideRain = double.Parse(sWords[2]);
										mInf.mAlertStopBeforeTime = double.Parse(sWords[3]);
										mInf.mAlertStopRain = double.Parse(sWords[4]);
										// m_csInf.mTimeAfterPeakEps = double.Parse(sWords[5]);
										iReadTiming++;
										break;
									}
									mInf.mLevIndex[mInf.mIndexNum] = double.Parse(sWords[0]);
									mInf.mVolIndex[mInf.mIndexNum] = double.Parse(sWords[1]);
									mInf.mIndexNum++;
									break;
								case 10: // mTimeStepOffAlert mTimeStepOnAlert
									if (sWords.Length != 2) return false;
									mInf.mTimeStepOffAlert = double.Parse(sWords[0]);
									mInf.mTimeStepOnAlert = double.Parse(sWords[1]);
									iReadTiming++;
									break;
								case 11: // mAveragingTime
									if (sWords.Length != 1) return false;
									mInf.mAveragingTime = double.Parse(sWords[0]);
									iReadTiming++;
									break;
								case 12: // mShortTargetTime
									if (sWords.Length != 1) return false;
									mInf.mShortTargetTime = double.Parse(sWords[0]);
									iReadTiming++;
									break;

								case 13: // mLongTargetTime
									if (sWords.Length != 1) return false;
									mInf.mLongTargetTime = double.Parse(sWords[0]);
									iReadTiming++;
									break;
							}

						}
					}
					fpParam.Close();
				}
				catch (Exception e)
				{
					McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Initialize", "FileStreamError:" + mInf.m_pmtfile + " Line:" + sLine);
					McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "Message:" + e.Message);
					McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "StackTrace:" + e.StackTrace);
					return false;

				}
			}
			return true;
		}
		/// <summary><para>method outline:</para>
		/// <para>演算実行中断中のプロパティ等情報設定</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = SetOnlineProperty(csCellMdlPropertyInfo)</para>
		/// </example>
		/// <param name="csCellMdlPropertyInfo">プロパティ情報</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>演算中断中にプロパティ、初期値等を変更して動作させ場合等に使用する</para>
		/// </remarks>
		public override bool SetOnlineProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
		{
			return true;
		}
	}
}
