﻿using CommonMP.HYSSOP.CoreImpl;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYMCO.Interface;
using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.CoreImpl.Data;

namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>KyotoUniv.Hydrology演算系モデルファクトリクラス</para>
	/// </summary>
	public class DamModelFactory : McBasicModelFactoryBase
	{
		/// <summary><para>method outline:</para>
		/// <para>デフォルトコンストラクタ</para>
		/// </summary>
		/// <param name="">無し</param>
		/// <returns>インスタンス</returns>
		/// <exception cref="">無し</exception>
		public DamModelFactory( )
		{
		}

		/// <summary><para>method outline:</para>
		/// <para>ファクトリ識別子を生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> HySID csFactID = CreateFactoryID( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>ファクトリ識別子</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override HySID CreateFactoryID()
		{
			return DamModelDefine.HYDROLOGY_MODEL_LIB;
		}

		/// <summary><para>method outline:</para>
		/// <para>演算モデルクラス生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McCalModel csCalModel = CreateCalModel(csModelKind) </para>
		/// </example>
		/// <param name="csModelKind">モデル種別識別子</param>
		/// <returns>演算モデルクラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override McCalModel CreateCalModel(HySObjectKind csModelKind)
		{
			McCalModel csCalModel = null;
			if (csModelKind == DamModelDefine.HYDROLOGY_DAM_KIND)
			{
				csCalModel = new DamModel();
			}
			else
			{
			}
			
			return csCalModel;
		}

		/// <summary><para>method outline:</para>
		/// <para>要素内演算中データクラス生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McCalInfo csCalModel = CreateCalInfo( csModelKind ) </para>
		/// </example>
		/// <param name="csModelKind">モデル種別識別子</param>
		/// <returns>要素内演算中データクラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override McCalInfo CreateCalInfo(HySObjectKind csModelKind)
		{
			McCalInfo csCalInfoData = null;
			
			if (csModelKind == DamModelDefine.HYDROLOGY_DAM_KIND)
			{
				csCalInfoData = new DamModelInfo();
			}
			else
			{
			}

			return csCalInfoData;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデルプロパティ表示／設定情報クラス生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McPropertyInfoRoot csPropertyInf = CreateModelPropertyInfo(csLibraryID, csModelKind ) </para>
		/// </example>
		/// <param name="csLibraryID">ライブラリー識別子</param>
		/// <param name="csModelKind">モデル種別識別子</param>
		/// <returns>モデルプロパティ表示／設定情報クラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override McCellModelPropertyIF CreateModelProperty(HySID csLibraryID, HySObjectKind csModelKind)
		{
			if (this.EqualFactory(csLibraryID) == false)
			{
				return null;
			}

			McCellModelPropertyInfo csRtnCellPrptyDt = null;

			if (csModelKind == DamModelDefine.HYDROLOGY_DAM_KIND)
			{
				csRtnCellPrptyDt = new McCellModelPropertyInfo(csLibraryID, csModelKind);
				csRtnCellPrptyDt.SetStepTime((double)600); // δT設定 60秒
				csRtnCellPrptyDt.AddInfoType("m_pmtfile", "ダム情報ファイル名", McDefine.ValKind.STRING);
				csRtnCellPrptyDt.SetInfo("m_pmtfile", "./ModelSample/DamSample.dat");

				{
					McTranInfoPattern csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
						DamModelDefine.IN_PATTERN_TOP,
						McTranInfoDefine.SINGLE_CELL_SERIAL, "上流端流入") as McTranInfoPattern;
					{
						// セル内変数設定
						HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
						{
							csCellChara.SetDataKind(0, "流量", HySDataKindDefine.QUANTITY_OF_WATER_FLOW, "m3/sec"); // 最初の変数は流量
						}
						csTrnPtn.SetCellDataCharacteristic(csCellChara);
						csTrnPtn.SetInterpolateType(HySDefine.InterpolateType.LINEAR); // 内挿方法設定
					}
					csRtnCellPrptyDt.AddReceivePattern(csTrnPtn);	// 受信可能パターンの追加

					// 第2パターン
					csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(DamModelDefine.IN_RAINDATA_POINT,
					McTranInfoDefine.SINGLE_CELL_SERIAL, "一様雨量") as McTranInfoPattern;
					{
						// セル内変数設定
						HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
						{
							csCellChara.SetDataKind(0, "単位時間雨量強度", HySDataKindDefine.SCALOR_RAIN_FALL, "mm/hour");
						}
						csTrnPtn.SetCellDataCharacteristic(csCellChara);
						csTrnPtn.SetInterpolateType(HySDefine.InterpolateType.LINEAR); // 内挿方法設定
					}
					csRtnCellPrptyDt.AddReceivePattern(csTrnPtn);	// 受信可能パターンの追加
				}
				// 送信パターンの設定
				{
					// 第1パターン
					McTranInfoPattern csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
						DamModelDefine.OUT_PATTERN_BOTTOM,
						McTranInfoDefine.SINGLE_CELL_SERIAL,
						"下流端出力") as McTranInfoPattern;
					{
						// セル内変数設定
						HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
						{
							csCellChara.SetDataKind(0, "流量", HySDataKindDefine.QUANTITY_OF_WATER_FLOW, "m3/sec");
						}
						csTrnPtn.SetCellDataCharacteristic(csCellChara);
					}
					csRtnCellPrptyDt.AddSendPattern(csTrnPtn);

					csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
					DamModelDefine.OUT_PATTERN_COND,
					McTranInfoDefine.SINGLE_CELL_SERIAL,
					"ダム状態出力") as McTranInfoPattern;
					{
						// セル内変数設定
						HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(3);
						{
							csCellChara.SetDataKind(0, "水位", HySDataKindDefine.WATER_LEVEL, "m");
							csCellChara.SetDataKind(1, "貯水量", HySDataKindDefine.WATER_VOLUME, "m3");
							csCellChara.SetDataKind(2, "ダム操作番号", HySDataKindDefine.ANY_VALUE, "");

						}
						csTrnPtn.SetCellDataCharacteristic(csCellChara);
					}

					csRtnCellPrptyDt.AddSendPattern(csTrnPtn);
				}
			}
			else
			{
				// Do nothing
			}

			return csRtnCellPrptyDt;
		}
		/// <summary><para>method outline:</para>
		/// <para>モデル初期化表示／設定情報生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McPropertyInfoRoot csInitialInf = CreateModelInitialInfo(csLibraryID, csModelKind ) </para>
		/// </example>
		/// <param name="csLibraryID">ライブラリー識別子</param>
		/// <param name="csModelKind">モデル種別識別子</param>
		/// <returns>初期情報クラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override McPropertyInfoRoot CreateModelInitialInfo(HySID csLibraryID, HySObjectKind csModelKind)
		{
			if (this.EqualFactory(csLibraryID) == false)
			{
				return null;
			}

			McInitialInfo csRtnDt = null;
			
			if (csModelKind == DamModelDefine.HYDROLOGY_DAM_KIND)
			{
				csRtnDt = new McInitialInfo(csLibraryID, csModelKind);
				csRtnDt.AddInfoType("m_wlev", "初期水位(m)", McDefine.ValKind.DOUBLE);
				csRtnDt.SetInfo("m_wlev", 280.0);
			}
			else
			{
				// Do Nothing
			}

			return csRtnDt;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデル情報提供</para>
		/// </summary>
		/// <example>
		/// <para>usage:</para>
		/// <para>HySDataLinkedList csModelInfoList = GetCalModelInfoList();</para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>モデル情報のリスト</returns>
		/// <exception cref="">無し</exception>
		/// <remarks>
		/// <para>remarks:</para>
		/// <para>モデル情報(McModelInfo)をリスト形式で返却する</para>
		/// </remarks>
		public override HySDataLinkedList GetCalModelInfoList()
		{
			McModelInfo csModelInfo=null;


    		csModelInfo = new McModelInfo(
					(HySID)this.GetFactoryID(),
					McModelLibraryDefine.DIVISION_CALCULATION_MODEL,
					McModelLibraryDefine.MODEL_CLASSIFICATION_CAL_DAM,
					DamModelDefine.HYDROLOGY_DAM_KIND,
					DamModelDefine.HYDROLOGY_DAM_NAME);
			csModelInfo.SetVersionInf("Ver1.0 ");
			csModelInfo.SetSummaryInf("ダムモデル");
			csModelInfo.SetCreatorInf("京都大学大学院 水文・水資源学研究室");
			// csModelInfo.SetIconName("Dam");
            csModelInfo.SetIconName("Lane");
			m_csCalModelInforList.AddLast(csModelInfo);

			return m_csCalModelInforList;
		}

	}
}


namespace CommonMP.HYMCO._SYSTEM_
{
	/// <summary><para>class outline:</para>
	/// <para>KyotoUniv.Hydrology演算系モデルファクトリクラス</para>
	/// </summary>
	public class HYMCO_MODEL_FACTORY_CREATOR_IMPLE : HYMCO_MODEL_FACTORY_CREATOR
	{
		/// <summary><para>method outline:</para>
		/// <para>コンストラクター</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> HYMCO_MODEL_FACTORY_CREATOR_IMPLE csFactCrt = new HYMCO_MODEL_FACTORY_CREATOR_IMPLE( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>生成インスタンス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public HYMCO_MODEL_FACTORY_CREATOR_IMPLE()
		{
		}
		/// <summary><para>method outline:</para>
		/// <para>演算モデルファクトリクラス生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McModelFactory csCalModelFactory = CreateFactory( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>演算モデルファクトリクラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public CommonMP.HYMCO.Interface.Model.McModelFactory CreateFactory()
		{
			return new KyotoUnivEngHywr.DamModelFactory();
		}
	}
}