﻿using System;
using System.Collections.Generic;
using System.Text;

using CommonMP.HYSSOP.Interface.HSData;
using CommonMP.HYSSOP.Interface.HSViewer;
using CommonMP.HYSSOP.CoreImpl;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;

using CommonMP.HYMCO.Interface;
using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.Interface.Controller;

using CommonMP.HYMCO.CoreImpl.Data;


namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>KyotoUnivHydrologyDamModel ダムデータクラス</para>
	/// </summary>
	[Serializable]
	public class DamModelInfo : McCalInfo
	{
		// 各種水位: すべて単位は [m]
		// 以下はパラメータファイルから読み込まれる諸表である
		
		/// <summary> パラメータファイル名</summary>
		public string m_pmtfile;

		// 水位（単位m)
		/// <summary> 現在の水位 [m] (state)</summary>
		public double mWlev;
		/// <summary>サーチャージ水位 (pmt) </summary>
		public double mWlevSurcharge;
		/// <summary>ただし書き操作開始水位 (pmt) </summary>
		public double mWlevIrregularRelease;
		/// <summary>常時満水位(pmt)</summary>
		public double mWlevHighest;
		/// <summary>確保水位 (pmt) </summary>
		public double mWlevMinimum;
		/// <summary>最低水位 (pmt)</summary>
		public double mWlevLowest;
		/// <summary>現在時刻の目標水位(state)</summary>
		public double mWlevTarget;

		// 流量（単位 m3/sec）
		/// <summary>洪水流量 (pmt) </summary>
		public double mQflood;
		/// <summary>最低流量 </summary>
		public double mQMinimum;

		/// <summary>ピーク時の流入量を記憶するための変数 (state) </summary>
		public double mQpeakIn;
		/// <summary>現時刻の流入量 (bnd) </summary>
		public double mQinCurrent;
		/// <summary>現時刻の流出量 (bnd) </summary>
		public double mQoutCurrent;
		/// <summary>次時刻の流入量 (bnd) </summary>
		public double mQinNext;
		/// <summary>次時刻の流出量 (bnd) </summary>
		public double mQoutNext;
		/// <summary> 放流に用いる流入基準値 (bnd) </summary>
		public double mQregularrelease;


		// 各種貯水量: 単位は [m3]
		/// <summary>現在の貯留量 (state) </summary>
		public double mVol;
		/// <summary>現在時刻の目標水位(state)</summary>
		public double mVolTarget;

		// 雨量(単位 mm/hour)
		/// <summary> 次時刻の雨量</summary>
		public double mRainNext;
		/// <summary> 次時刻の雨量</summary>
		public double mRainCurrent;


		/// <summary>洪水期間フラッグ YES: 洪水期間中, NO: 非洪水期間(state)</summary>
		public bool mFloodPeriodFlag;
		/// <summary>使用する制限水位の数</summary>
		public int mLimitedNum;
		/// <summary>制限水位 (pmt) </summary>
		public double[] mWlevLimited;
		/// <summary>制限水位のときの貯水量 (from pmt) </summary>
		public double[] mVolLimited;
		/// <summary>洪水期間開始日 (pmt) </summary>
		public HySTime[] mFloodStartDate;
		/// <summary>洪水期間終了日 (pmt) </summary>
		public HySTime[] mFloodEndDate;

		/// <summary> 放流調整の区分数</summary>
		public long mQmaxChangeRateNum;
		/// <summary> 放流量の増加上限</summary>
		public double[] mQmaxChangeRate = new double[10];
		/// <summary> 放流量増加上限の区分値</summary>
		public double[] mQmaxChangeFlowRate = new double[10];
		
		/// <summary>予備放流水位 (pmt)</summary>
		public double mWlevPreRelease;
		/// <summary>最大予備放流量 (pmt) </summary>
		public double mQmaxprerelease;
		/// <summary>実際の予備放流量 (state) </summary>
		public double mQprerelease;
		/// <summary>予備放流フラッグ YES: 予備放流中, NO: 予備放流終了(state)</summary>
		public bool mPreReleaseFlagCurrent, mPreReleaseFlagNext;
		/// <summary> 予備放流に入る判断に用いる過去の雨量時間</summary>
		public double mPreReleaseDecideBeforeTime;   // (pmt)
		/// <summary> 予備放流に入る判断に用いる過去の降雨強度</summary>
		public double mPreReleaseDecideBeforeRain;		// (pmt)
		/// <summary> 予備放流に入る判断に用いる将来の雨量時間</summary>
		public double mPreReleaseDecideAfterTime;	// (pmt)
		/// <summary> 予備放流に入る判断に用いる将来の降雨強度</summary>
		public double mPreReleaseDecideAfterRain;	// (pmt)
		/// <summary> 予備放流を行うのに必要な準備時間</summary>
		public double mPreReleasePrepareTime;  // (pmt)
		/// <summary>予備放流実施を決めた時刻(state) </summary>
		public HySTime mPreReleaseDecidedTime; 

		/// <summary> 水位貯留量関係の区分数</summary>
		public long mIndexNum;
		/// <summary> 水位貯留量関係-貯留量インデックス(m3)</summary>
		public double[] mVolIndex = new double[100];
		/// <summary> 水位貯留量関係-水位インデックス(m)</summary>
		public double[] mLevIndex = new double[100];

		/// <summary> 一定率一定量放流の流量係数</summary>
		public double mQFlowParameter;
		/// <summary> 一定率一定量放流に移る流量（0なら一定量放流）(m3/sec)</summary>
		public double mQFlowRate;

		/// <summary>洪水警戒体制フラッグ YES: 洪水警戒体制中, NO: 解除中(state)</summary>
		public bool mAlertFlagCurrent, mAlertFlagNext;
		/// <summary> 洪水警戒態勢に入る判断に用いる過去の雨量時間</summary>
		public double mAlertDecideBeforeTime;	// (pmt)
		/// <summary> 洪水警戒態勢に入る判断に用いる将来の雨量時間</summary>
		public double mAlertDecideAfterTime;	// (pmt)
		/// <summary> 洪水警戒態勢に入る判断に用いる降雨強度</summary>
		public double mAlertDecideRain;	// (pmt)
		/// <summary> 洪水警戒態勢を解除する判断に用いる過去の雨量時間</summary>
		public double mAlertStopBeforeTime;	// (pmt)
		/// <summary> 洪水警戒態勢を解除する判断に用いる降雨強度</summary>
		public double mAlertStopRain;		// (pmt)

		/// <summary>通常時の計算時間間隔 [sec] (pmt)</summary>
		public double mTimeStepOffAlert;
		/// <summary>洪水警戒態勢時の計算時間間隔 [sec](pmt)</summary>
		public double mTimeStepOnAlert;
		
		/// <summary> RegularRelease は、mAveragingTime 間の流入量の平均値を求める。 (pmt)</summary>
		public double mAveragingTime;

		/// <summary> ダムの短期調整基準時間(sec)</summary>
		public double mShortTargetTime;
		
		/// <summary> ダムの長期調整基準時間(sec)</summary>
		public double mLongTargetTime;


		/// <summary>ピーク前後を示すフラッグ  YES: ピーク後, NO: ピーク前 (state)</summary>
		public bool mPeakFlag;
		/// <summary> 次時刻の操作の状況を示す </summary>
		public int mCondNext;
		
		/// <summary> シミュレーション開始日 (state)</summary>
		public HySTime mSimStartDate;

		/// <summary>過去の降雨 (mm/h) </summary>
		public double[] mPastRain;
		/// <summary>過去の降雨シリーズの時刻（currentをゼロとして過去にさかのぼった時間）を記憶するための配列 </summary>
		public double[] mPastTime;
		/// <summary>過去の流入量 </summary>
		public double[] mPastInflow;
		/// <summary> 過去の降雨を記録する配列の数</summary>
		public long numberOfPastRainSteps;

		/// <summary><para>method outline:</para>
		/// <para>自己複製</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> Clone( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>同一内容で　別インスタンス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>収束演算時、特定の時刻に状態を戻す場合等に使用する</para>
		/// </remarks>
		public override McCalInfo Clone()
		{
			DamModelInfo csRtn = new DamModelInfo();
			csRtn.CopyInfo(this);
			return csRtn;
		}
		/// <summary><para>method outline:</para>
		/// <para>引数で与えられた情報を自分にコピーを行う</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> bool bRtn = CopyInfo(csOrgInfo) </para>
		/// </example>
		/// <param name="csOrgInfo">コピー元情報</param>
		/// <returns> bool true :正常 , false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override bool CopyInfo(McCalInfo csOrgInfo)
		{
			DamModelInfo csOrgDt = (DamModelInfo)csOrgInfo;
			
			return true;
		}

	}

}
